'use strict';

const webpack = require('webpack');

const paths = require('../config/paths');
const insertPreloaders = require('./utils/insertPreloaders');
const appendBabelPlugins = require('./utils/appendBabelPlugins');
const shouldApplyAntIconLoader = require('./utils/shouldApplyAntIconLoader');
const {
  injectStylePlaceholderPlugins,
  injectStylePlaceholderLoaders,
} = require('./utils/injectLazyPlaceholder');

const preloaders = [];

if (shouldApplyAntIconLoader()) {
  preloaders.push({
    loader: require.resolve('webpack-ant-icon-loader'),
    enforce: 'pre',
    include: [require.resolve('@ant-design/icons/lib/dist')],
  });
}

// for template/src
const getBabelPlugins = webpackEnv => {
  const isEnvProduction = webpackEnv === 'production';

  return [
    isEnvProduction && [
      require.resolve('babel-plugin-transform-imports'),
      {
        lodash: {
          transform: 'lodash/${member}',
          preventFullImport: true,
        },
        antd: {
          transform(importName) {
            return `antd/es/${importName.replace(/([a-z0–9])([A-Z])/g, "$1-$2").toLowerCase()}`;
          },
          preventFullImport: true,
        },
        recompose: {
          transform: 'recompose/${member}',
          preventFullImport: true,
        },
      },
    ],

    isEnvProduction && [
      require.resolve('babel-plugin-custom-import-path-transform'),
      {
        transformImportPath: paths.customTransformImportPath,
      },
    ],
  ].filter(Boolean);
};

// for node_modules
const getNodeModulesBabelPlugins = webpackEnv => {
  const isEnvProduction = webpackEnv === 'production';

  return [
    isEnvProduction && [
      require.resolve('babel-plugin-custom-import-path-transform'),
      {
        transformImportPath: paths.customTransformImportPath,
      },
    ],
  ].filter(Boolean);
};

const transformRules = (rules, webpackEnv) => {
  injectStylePlaceholderLoaders(rules);

  insertPreloaders(rules, preloaders);

  appendBabelPlugins(rules, getBabelPlugins(webpackEnv));

  appendBabelPlugins(
    rules,
    getNodeModulesBabelPlugins(webpackEnv),
    rule => rule.include !== paths.appSrc
  );

  return rules;
};

const transformPlugins = plugins => {
  const configNameRegex = new RegExp(
    `config.${process.env.REACT_APP_BUILD_ENV}.(js|ts)`
  );

  plugins.push(
    new webpack.ContextReplacementPlugin(/src\/config/, configNameRegex)
  );

  injectStylePlaceholderPlugins(plugins);

  return plugins;
};

/**
 * Why? With react-scripts@5.0.0 comes this issue https://github.com/facebook/create-react-app/pull/11752,
 * so until it's solved, it's better to ignore those irrelevant warnings and to have cleaner console output.
 */
function ignoreFailedParsedSourceMaps(webpackConfig) {
  const failedParsedSourceMaps = /Failed to parse source map/;
  if (Array.isArray(webpackConfig.ignoreWarnings)) {
    webpackConfig.ignoreWarnings.push(failedParsedSourceMaps);
  } else {
    webpackConfig.ignoreWarnings = [failedParsedSourceMaps];
  }
}

/**
 *
 * @param {ReturnType<typeof import('../../config/webpack.config.js')>} webpackConfig
 * @param {'development' | 'production'} webpackEnv
 * @returns {ReturnType<typeof import('../../config/webpack.config.js')>} (it's almost accurate return type)
 */
module.exports = (webpackConfig, webpackEnv) => {
  webpackConfig.module.rules = transformRules(
    webpackConfig.module.rules,
    webpackEnv
  );

  webpackConfig.plugins = transformPlugins(webpackConfig.plugins, webpackEnv);

  ignoreFailedParsedSourceMaps(webpackConfig);

  return webpackConfig;
};
