'use strict';

const MiniCssExtractPlugin = require('mini-css-extract-plugin');
const HtmlWebpackInjectStringPlugin = require('html-webpack-inject-string-plugin');

const findOneOf = require('./findOneOf');

const LAZY_STYLES_PLACEHOLDER_ID =
  process.env.LAZY_STYLES_PLACEHOLDER_ID || 'lazy-styles-placeholder';

// To ensure correct ordering of lazy loaded styles together with Fela,
// we need to load those styles to some place in <head>.
// Upcoming code ensures that style-loader and MiniCSSExtractPlugin load
// styles to a placeholder identified by LAZY_STYLES_PLACEHOLDER_ID.
// Fela will add its styles at the end of <head> and thus the order will
// be correct.

function injectStylePlaceholderPlugins(plugins = []) {
  const pluginIndex = plugins.findIndex(
    plugin => plugin instanceof MiniCssExtractPlugin
  );

  if (pluginIndex >= 0) {
    plugins[pluginIndex] = new MiniCssExtractPlugin({
      // start - These options are copied from official webpack.config.js
      filename: 'static/css/[name].[contenthash:8].css',
      chunkFilename: 'static/css/[name].[contenthash:8].chunk.css',
      // end
      insert: `#${LAZY_STYLES_PLACEHOLDER_ID}`,
    });
  }

  plugins.push(
    new HtmlWebpackInjectStringPlugin({
      prepend: true,
      search: '</head>',
      inject: `<style id="${LAZY_STYLES_PLACEHOLDER_ID}"></style>`,
    })
  );
}

function injectStylePlaceholderLoaders(rules) {
  const oneOf = findOneOf(rules);

  const styleLoaders = oneOf.filter(loader => Array.isArray(loader.use));

  const targetLocations = [];
  const styleLoaderPath = require.resolve('style-loader');

  // Find 'style-loader' in style loaders
  styleLoaders.forEach((loader, loaderIndex) => {
    const targetIndex = loader.use.findIndex(
      subLoader => subLoader === styleLoaderPath
    );

    if (targetIndex >= 0) {
      targetLocations.push([loaderIndex, targetIndex]);
    }
  });

  // Delete 'style-loader' from style loaders
  for (const [loaderIndex, targetIndex] of targetLocations) {
    const loader = styleLoaders[loaderIndex].use[targetIndex];
    styleLoaders[loaderIndex].use[targetIndex] = {
      loader,
      options: {
        insert: `#${LAZY_STYLES_PLACEHOLDER_ID}`,
      },
    };
  }
}

module.exports = {
  injectStylePlaceholderPlugins,
  injectStylePlaceholderLoaders,
};
