'use strict';

const madge = require('madge');
const chalk = require('chalk');
const { performance } = require('perf_hooks');
const fs = require('fs-extra');
const paths = require('../../config/paths');

const availableArguments = {
  SILENT: '--silent',
  IGNORE_INDEX_FILES: '--ignore-index-files',
};

function printCircular(circular = [], { ignoreIndexFiles }) {
  console.log('');
  if (circular.length === 0) {
    console.log(chalk.green.bold('  No circular dependencies found.\n'));
  } else {
    console.log(
      chalk.red.bold(`  Found ${circular.length} circular dependencies:\n`)
    );

    if (ignoreIndexFiles) {
      const indexRegex = new RegExp('index.*');
      circular = circular.map(moduleDependencies => {
        return moduleDependencies.filter(dependency => {
          const fileNames = dependency.split('/');
          const lastFileName = fileNames[fileNames.length - 1];
          return !indexRegex.test(lastFileName);
        });
      });
    }

    circular.forEach((moduleDependencies, index) => {
      console.log(chalk.bold(`  #${index + 1}`));
      for (const dependency of moduleDependencies) {
        console.log(`   ${chalk.cyan(dependency)}`);
      }
      console.log('');
    });
  }
}

function logDuration(res, t0, t1) {
  const duration = Math.round(t1 - t0);
  const files = Object.keys(res.obj());

  console.log(
    chalk.dim(
      `  Processed ${files.length} file${
        files.length === 1 ? '' : 's'
      } in ${duration}ms.`
    )
  );
}

async function performanDependenciesAnalysis({ options, notSilent }) {
  try {
    const t0 = performance.now();
    const res = await madge(paths.appSrc, options);
    const t1 = performance.now();

    if (notSilent) {
      logDuration(res, t0, t1);
    }

    return res;
  } catch (e) {
    let msg = chalk.red.bold(`Failed to analyze dependencies:\n`);
    msg += chalk.dim(`\ninput options: ${JSON.stringify(options, null, 2)}\n`);

    console.error(msg);
    console.error(e);

    return null;
  }
}

function getOptions() {
  if (fs.existsSync(paths.madgerc)) {
    return fs.readJSONSync(paths.madgerc);
  }

  return {
    fileExtensions: ['ts', 'tsx'],
    detectiveOptions: {
      ts: {
        skipTypeImports: true,
      },
    },
    tsConfig: paths.tsconfig,
  };
}

(async function analyzeDependencies() {
  const args = process.argv.slice(2);
  const options = getOptions();
  const notSilent = !args.includes(availableArguments.SILENT);
  const ignoreIndexFiles = args.includes(availableArguments.IGNORE_INDEX_FILES);

  if (notSilent) {
    console.log(chalk.bold(`Analyzing module dependencies...\n`));
    console.log(chalk.dim(`  scope: ${paths.appSrc}`));
    console.log('');
  }

  const res = await performanDependenciesAnalysis({ options, notSilent });

  if (!res) {
    process.exit(1);
  }

  const circular = res.circular();

  if (notSilent) {
    printCircular(circular, { ignoreIndexFiles });
  }

  process.exit(circular.length === 0 ? 0 : 1);
})();
