'use strict';

const fs = require('fs-extra');

const getUnmarkFileContent = require('./utils/getUnmarkFileContent');

/**
 * @param {string} name
 * @param {'begin' | 'end' | '(begin|end)'} [positionType]
 * @returns {RegExp}
 */
function createCommentRegex(name, positionType = '(begin|end)') {
  return new RegExp(`// @use-${name}-${positionType}`, 'gm');
}

/**
 * @param {string} file
 */
function getFileSuffix(file) {
  return file.split('.').pop();
}

class OptionalModule {
  /**
   * @param {Object} props
   * @param {string} props.name e.g. 'auth-module'
   * @param {boolean} props.useModule
   * @param {Object} props.appPackage
   * @param {Promise<{
   *    files?: string[];
   *    directories?: string[]
   *    packageJson?: {
   *        dependencies?: string[]
   *    }
   * }>} props.dependencies
   * @param {(content: string, fileSuffix: string) => Promise<string>} props.prettierFormatter
   */
  constructor({
    name,
    useModule,
    appPackage,
    templatePackage,
    dependencies,
    prettierFormatter,
  }) {
    this.useModule = useModule;
    this.blockMarker = {
      begin: createCommentRegex(name, 'begin'),
      end: createCommentRegex(name, 'end'),
      beginOrEnd: createCommentRegex(name),
    };
    this.appPackage = appPackage;
    this.templatePackage = templatePackage;
    this.dependencies = dependencies;
    this.prettierFormatter = prettierFormatter;
  }

  async resolveDependencies() {
    this.dependencies = await this.dependencies;
  }

  /**
   * If the module won't be used, then remove all of its package.json dependencies
   */
  async transformPackageJson() {
    this.dependencies.packageJson.dependencies.forEach(dependency => {
      delete this.appPackage.dependencies[dependency];
      delete this.templatePackage.dependencies[dependency];
    });
  }

  /**
   * If the module won't be used, then remove all of its dependency directories.
   */
  async removeDirectories() {
    await Promise.all(this.dependencies.directories.map(dir => fs.remove(dir)));
  }

  /**
   * If the module will be used, then just remove the comment markers (e.g. `// @use-auth-module-begin` and `// @use-auth-module-end`).
   */
  async removeComments() {
    for (const file of this.dependencies.files) {
      let content = fs.readFileSync(file, 'utf8');

      content = content
        .split('\n')
        .filter(line => !line.match(this.blockMarker.beginOrEnd))
        .join('\n');

      fs.writeFileSync(
        file,
        this.prettierFormatter(content, getFileSuffix(file))
      );
    }
  }

  /**
   * If the module won't be used, then go through each file provided in dependencies:
   * - if given file contain comment marker, then just remove the marked code
   * - otherwise remove the whole file
   */
  async processFilesContent() {
    const removingTasks = [];

    for (const file of this.dependencies.files) {
      let content = fs.readFileSync(file, 'utf8');

      if (content.match(this.blockMarker.begin) === null) {
        // if file doesn't contain @use-auth-module-*, but it's placed
        // among a module dependencies, then remove the whole file
        removingTasks.push(fs.remove(file));
      } else {
        // otherwise just remove code within those comments
        content = getUnmarkFileContent(this.blockMarker, content);

        fs.writeFileSync(
          file,
          this.prettierFormatter(content, getFileSuffix(file))
        );
      }
    }

    await Promise.all(removingTasks);
  }
}

module.exports = OptionalModule;
